#ifndef AST_STR_DEFS_H
#define AST_STR_DEFS_H

#define AST_STR_DEFS \
	X(kind) \
	X(flags) \
	X(lineno) \
	X(children) \
	X(name) \
	X(docComment) \
	X(endLineno) \
	X(params) \
	X(uses) \
	X(stmts) \
	X(returnType) \
	X(extends) \
	X(implements) \
	X(expr) \
	X(var) \
	X(offset) \
	X(label) \
	X(depth) \
	X(dim) \
	X(prop) \
	X(class) \
	X(args) \
	X(const) \
	X(left) \
	X(right) \
	X(value) \
	X(key) \
	X(default) \
	X(cond) \
	X(declares) \
	X(traits) \
	X(adaptations) \
	X(method) \
	X(insteadof) \
	X(alias) \
	X(prefix) \
	X(true) \
	X(false) \
	X(try) \
	X(catches) \
	X(finally) \
	X(type) \
	X(init) \
	X(loop) \

#endif
